/*grating.c - generate a sine-wave grating
  by Matthew Belmonte
  Copyright (c) 1998 Howard Hughes Medical Institute

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  If you find this program useful, please send mail to Matthew Belmonte.
  <mkb4@Cornell.edu>.  If you base a publication on data processed by this
  program, please notify Matthew Belmonte and acknowledge him in your
  publication.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <errno.h>
#include <gl/image.h>
#define PI 3.1415926535897932384626433832795028841971693993751058209749445923078

/*Draw a d x d patch of a sine-wave pattern, with spatial period 'sper' pixels,
  spatial phase 'phase' radians, and orientation 'ori' radians from vertical.
  Place the result in a file with the given name.*/
void draw_sinewave(d, sper, phase, ori, filename)
int d;
double sper, phase, ori;
char *filename;
  {
  register int x, y;	/*rotated coordinates*/
  double sine, cosine,	/*sin(ori), cos(ori) - for inverse rotation*/
	origin;		/*middle of the image - for calculating spatial phase*/
  IMAGE *img;		/*image file*/
  unsigned short *row;	/*pointer to dynamically allocated row buffer*/
/*calculate derived parameters*/
  sine = sin(ori);
  cosine = cos(ori);
  origin = (d-1)/2.0;
/*allocate buffer*/
  row = (unsigned short *)calloc(d, sizeof(*row));
  if(row == (unsigned short *)0)
    {
    perror((char *)0);
    exit(errno);
    }
/*create image file*/
  img = iopen(filename, "w", RLE(1), 3, d, d, 1);
  for(y = 0; y != d; y++)
    {
    for(x = 0; x != d; x++)
      row[x] = (unsigned short int)(0.5+127.5*(1.0+sin(phase+2.0*PI*((x-origin)*cosine+(y-origin)*sine)/sper)));
    putrow(img, row, y, 0);
    }
  iclose(img);
  free(row);
  }

void main(argc, argv)
int argc;
char **argv;
  {
  int d;
/*check usage*/
  if((argc != 6) || ((d=atoi(argv[1])) < 1))
    {
    fprintf(stderr, "usage: %s <width> <spatial period> <phase> <orientation> <file name>\n", *argv);
    exit(1);
    }
  draw_sinewave(d, fabs(atof(argv[2])), 2.0*PI*atof(argv[3])/360.0, 2.0*PI*atof(argv[4])/360.0, argv[5]);
  exit(0);
  }
